@@ -1,6 +1,7 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
+#include "graphs2d/qabstractseries_p.h"
 #include <QtGraphs/qabstractseries.h>
 #include <QtGraphs/qbarseries.h>
 #include <QtGraphs/qbarset.h>
@@ -262,6 +263,10 @@ QT_BEGIN_NAMESPACE
         \li barSelected
         \li This value is true when the bar is selected, meaning that the bar index
         is in \l{QBarSet::selectedBars}.
+    \row
+        \li int
+        \li barIndex
+        \li Index of the bar, from 0 to the amount of bars - 1. [since 6.9]
     \endtable
 
     To use any of these, add property with the defined name into your custom component.
@@ -308,6 +313,10 @@ QT_BEGIN_NAMESPACE
         \li barSelected
         \li This value is true when the bar is selected, meaning that the bar index
         is in \l{BarSet::selectedBars}.
+    \row
+        \li int
+        \li barIndex
+        \li Index of the bar, from 0 to the amount of bars - 1. [since 6.9]
     \endtable
 
     To use any of these, add property with the defined name into your custom component.
@@ -377,17 +386,6 @@ QT_BEGIN_NAMESPACE
     Returns the bar set at \a index. Returns null if the index is not valid.
 */
 
-/*!
-    \qmlmethod BarSet BarSeries::append(string label, VariantList values)
-    Adds a new bar set with \a label and \a values to the index. \a values is
-    a list of real values.
-
-    For example:
-    \code
-        myBarSeries.append("set 1", [0, 0.2, 0.2, 0.5, 0.4, 1.5, 0.9]);
-    \endcode
-*/
-
 /*!
     \qmlmethod BarSet BarSeries::insert(int index, string label, VariantList values)
     Adds a new bar set with \a label and \a values to \a index. \a values can be a list
@@ -400,6 +398,18 @@ QT_BEGIN_NAMESPACE
     \sa append()
 */
 
+/*!
+    \qmlmethod bool BarSeries::append(BarSet barset)
+
+    Adds a set of bars specified by \a barset to the bar series and takes
+    ownership of it. If the set is null or it already belongs to the series,
+    it will not be appended.
+
+    Returns \c true if appending succeeded.
+
+    \sa insert()
+*/
+
 /*!
     \qmlmethod bool BarSeries::remove(BarSet barset)
     Removes the bar set specified by \a barset from the series. Returns \c true if successful,
@@ -453,6 +463,31 @@ QT_BEGIN_NAMESPACE
     to match. Returns false if any of the bar set in \a sets are invalid.
 */
 
+/*!
+    \qmlsignal BarSeries::clicked(int index, BarSet barset)
+    This signal is emitted when the user clicks or taps the bar specified by \a index
+    in the bar set specified by \a barset.
+*/
+
+/*!
+    \qmlsignal BarSeries::doubleClicked(int index, BarSet barset)
+    This signal is emitted when the user double-clicks or double-taps the bar specified
+    by \a index in the bar set specified by \a barset.
+    This signal always occurs after \l clicked.
+*/
+
+/*!
+    \qmlsignal BarSeries::pressed(int index, BarSet barset)
+    This signal is emitted when the user clicks or taps the bar specified by \a index
+    in the bar set specified by \a barset and holds down the mouse button or gesture.
+*/
+
+/*!
+    \qmlsignal BarSeries::released(int index, BarSet barset);
+    This signal is emitted when the user releases a previously pressed mouse button
+    or gesture on the bar specified by \a index in the bar set specified by \a barset.
+*/
+
 /*!
     \internal
 */
@@ -503,8 +538,18 @@ QList<QColor> QBarSeries::seriesColors() const
 void QBarSeries::setSeriesColors(const QList<QColor> &newSeriesColors)
 {
     Q_D(QBarSeries);
-    if (d->m_seriesColors == newSeriesColors)
+    for (const QColor &color : newSeriesColors) {
+        if (!color.isValid()) {
+            qCWarning(lcProperties2D, "QBarSeries::setSeriesColors. Tried to use invalid color value.");
+            break;
+        }
+    }
+
+    if (d->m_seriesColors == newSeriesColors) {
+        qCDebug(lcProperties2D) << "QBarSeries::setSeriesColors. Set value of" << newSeriesColors << "is the same as it already was.";
         return;
+    }
+
     d->m_seriesColors = newSeriesColors;
     emit seriesColorsChanged();
     emit update();
@@ -535,8 +580,18 @@ QList<QColor> QBarSeries::borderColors() const
 void QBarSeries::setBorderColors(const QList<QColor> &newBorderColors)
 {
     Q_D(QBarSeries);
-    if (d->m_borderColors == newBorderColors)
+    for (const QColor &color : newBorderColors) {
+        if (!color.isValid()) {
+            qCWarning(lcProperties2D, "QBarSeries::setBorderColors. Tried to use invalid color value.");
+            break;
+        }
+    }
+
+    if (d->m_borderColors == newBorderColors) {
+        qCDebug(lcProperties2D) << "QBarSeries::setBorderColors. Set value of:" << newBorderColors << "is the same than it already was.";
         return;
+    }
+
     d->m_borderColors = newBorderColors;
     emit borderColorsChanged();
     emit update();
@@ -549,6 +604,8 @@ void QBarSeries::setBarsType(QBarSeries::BarsType type)
         d->m_barsType = type;
         emit barsTypeChanged(type);
         emit update();
+    } else {
+        qCDebug(lcProperties2D) << "QBarSeries::setBarsType. Set value of:" << type << "is the same than it already was.";
     }
 }
 
@@ -567,6 +624,9 @@ void QBarSeries::setBarWidth(qreal width)
     if (d->barWidth() != width) {
         d->setBarWidth(width);
         emit barWidthChanged();
+    } else {
+        qCDebug(lcProperties2D, "QBarSeries::setBarWidth. Set value of: %f is the same than it already was.",
+                width);
     }
 }
 
@@ -595,6 +655,7 @@ bool QBarSeries::append(QBarSet *set)
         set->setParent(this);
         QObject::connect(set, &QBarSet::update, this, &QBarSeries::update);
         emit barsetsAdded(sets);
+        emit barSetsChanged();
         emit countChanged();
         emit update();
     }
@@ -615,6 +676,7 @@ bool QBarSeries::remove(QBarSet *set)
         set->setParent(0);
         QObject::disconnect(set, &QBarSet::update, this, &QBarSeries::update);
         emit barsetsRemoved(sets);
+        emit barSetsChanged();
         emit countChanged();
         emit update();
         delete set;
@@ -639,6 +701,7 @@ bool QBarSeries::take(QBarSet *set)
         sets.append(set);
         QObject::disconnect(set, &QBarSet::update, this, &QBarSeries::update);
         emit barsetsRemoved(sets);
+        emit barSetsChanged();
         emit countChanged();
         emit update();
     }
@@ -664,6 +727,7 @@ bool QBarSeries::append(const QList<QBarSet *> &sets)
     }
 
     emit barsetsAdded(sets);
+    emit barSetsChanged();
     emit countChanged();
     emit update();
     return true;
@@ -683,6 +747,7 @@ bool QBarSeries::insert(qsizetype index, QBarSet *set)
         sets.append(set);
         QObject::connect(set, &QBarSet::update, this, &QBarSeries::update);
         emit barsetsAdded(sets);
+        emit barSetsChanged();
         emit countChanged();
         emit update();
     }
@@ -699,6 +764,7 @@ void QBarSeries::clear()
     bool success = d->remove(sets);
     if (success) {
         emit barsetsRemoved(sets);
+        emit barSetsChanged();
         emit countChanged();
         for (QBarSet *set : sets) {
             QObject::disconnect(set, &QBarSet::update, this, &QBarSeries::update);
@@ -723,8 +789,8 @@ void QBarSeries::replace(qsizetype index, QBarSet *set)
     if (index < 0)
         index = 0;
 
-    delete d->m_barSets[index];
-    d->m_barSets[index] = set;
+    remove(d->m_barSets[index]);
+    d->insert(index, set);
 
     QList<QBarSet *> sets;
     sets.append(set);
@@ -771,9 +837,7 @@ void QBarSeries::removeMultiple(qsizetype index, qsizetype count)
 {
     Q_D(QBarSeries);
 
-    if (index + count >= d->m_barSets.size())
-        return;
-    if (index < 0 || count < 0)
+    if (index < 0 || count < 1 || index + count > d->m_barSets.size())
         return;
 
     for (qsizetype i = index; i < index + count; ++i)
@@ -811,8 +875,8 @@ bool QBarSeries::replace(QBarSet *oldValue, QBarSet *newValue)
 
     for (qsizetype i = 0; i < d->m_barSets.size(); ++i) {
         if (d->m_barSets[i] == oldValue) {
-            delete d->m_barSets[i];
-            d->m_barSets[i] = newValue;
+            remove(d->m_barSets[i]);
+            d->insert(i, newValue);
 
             QList<QBarSet *> sets;
             sets.append(newValue);
@@ -838,15 +902,15 @@ bool QBarSeries::replace(const QList<QBarSet *> &sets)
             return false;
     }
 
-    for (const auto set : d->m_barSets) {
-        delete set;
+    for (const auto set : std::as_const(d->m_barSets)) {
+        remove(set);
     }
 
     for (const auto set : sets) {
         QObject::connect(set, &QBarSet::update, this, &QBarSeries::update);
     }
 
-    d->m_barSets = sets;
+    d->append(sets);
     emit barsetsReplaced(sets);
 
     return true;
@@ -880,6 +944,8 @@ void QBarSeries::setLabelsVisible(bool visible)
         d->setLabelsVisible(visible);
         emit labelsVisibleChanged(visible);
         emit update();
+    } else {
+        qCDebug(lcProperties2D) << "QBarSeries::setLabelsVisible. Label visibility is already set to:" << visible;
     }
 }
 
@@ -900,6 +966,9 @@ void QBarSeries::setLabelsFormat(const QString &format)
         d->setLabelsDirty(true);
         emit labelsFormatChanged(format);
         emit update();
+    } else {
+        qCDebug(lcProperties2D) << "QBarSeries::setLabelsFormat. Format is already set to value:"
+                                << format;
     }
 }
 
@@ -917,6 +986,9 @@ void QBarSeries::setLabelsMargin(qreal margin)
         d->setLabelsDirty(true);
         emit labelsMarginChanged(margin);
         emit update();
+    } else {
+        qCDebug(lcProperties2D, "QBarSeries::setLabelsMargin. Margin is already set to value: %f",
+                margin);
     }
 }
 
@@ -934,6 +1006,9 @@ void QBarSeries::setLabelsAngle(qreal angle)
         d->setLabelsDirty(true);
         emit labelsAngleChanged(angle);
         emit update();
+    } else {
+        qCDebug(lcProperties2D, "QBarSeries::setLabelsAngle. Label angle is already set to value: %f",
+                angle);
     }
 }
 
@@ -950,6 +1025,9 @@ void QBarSeries::setLabelsPosition(QBarSeries::LabelsPosition position)
         d->m_labelsPosition = position;
         emit labelsPositionChanged(position);
         emit update();
+    } else {
+        qCDebug(lcProperties2D) << "QBarSeries::setLabelsPosition. Position is already set to:"
+                                << position;
     }
 }
 
@@ -967,6 +1045,9 @@ void QBarSeries::setLabelsPrecision(int precision)
         d->setLabelsDirty(true);
         emit labelsPrecisionChanged(precision);
         emit update();
+    } else {
+        qCDebug(lcProperties2D, "QBarSeries::setLabelsPrecision. Precision is already set to: %d",
+                precision);
     }
 }
 
@@ -985,8 +1066,12 @@ QQmlComponent *QBarSeries::barDelegate() const
 void QBarSeries::setBarDelegate(QQmlComponent *newBarDelegate)
 {
     Q_D(QBarSeries);
-    if (d->m_barDelegate == newBarDelegate)
+    if (d->m_barDelegate == newBarDelegate) {
+        qCDebug(lcProperties2D) << "QBarSeries::setBarDelegate. BarDelegate is already set to:"
+                                << newBarDelegate;
         return;
+    }
+
     d->m_barDelegate = newBarDelegate;
     d->m_barDelegateDirty = true;
     emit barDelegateChanged();
@@ -997,7 +1082,7 @@ void QBarSeries::setBarDelegate(QQmlComponent *newBarDelegate)
 void QBarSeries::selectAll()
 {
     Q_D(QBarSeries);
-    for (auto s : d->m_barSets) {
+    for (auto s : std::as_const(d->m_barSets)) {
         s->selectAllBars();
     }
 }
@@ -1006,7 +1091,7 @@ void QBarSeries::selectAll()
 void QBarSeries::deselectAll()
 {
     Q_D(QBarSeries);
-    for (auto s : d->m_barSets) {
+    for (auto s : std::as_const(d->m_barSets)) {
         s->deselectAllBars();
     }
 }
@@ -1014,9 +1099,14 @@ void QBarSeries::deselectAll()
 void QBarSeries::componentComplete()
 {
     for (auto *child : children()) {
-        if (auto bs = qobject_cast<QBarSet *>(child))
+        if (auto bs = qobject_cast<QBarSet *>(child)) {
             append(bs);
+            qCDebug(lcSeries2D) << "append barset" << bs << "to barseries. barset values:" << bs->values();
+        }
     }
+
+    qCDebug(lcEvents2D) << "QBarSeries::componentComplete.";
+
     QAbstractSeries::componentComplete();
 }
 
@@ -1057,16 +1147,17 @@ void QBarSeries::setBarDelegateDirty(bool dirty)
 }
 
 QBarSeriesPrivate::QBarSeriesPrivate()
-    : m_barWidth(0.5) // Default value is 50% of category width
-      , m_labelsVisible(false)
-      , m_visible(true)
-      , m_blockBarUpdate(false)
-      , m_labelsFormat()
-      , m_labelsMargin(0)
-      , m_labelsAngle(0)
-      , m_labelsPrecision(6)
-      , m_labelsDirty(true)
-      , m_barDelegateDirty(false)
+    : QAbstractSeriesPrivate(QAbstractSeries::SeriesType::Bar)
+    , m_barWidth(0.5) // Default value is 50% of category width
+    , m_labelsVisible(false)
+    , m_visible(true)
+    , m_blockBarUpdate(false)
+    , m_labelsFormat()
+    , m_labelsMargin(0)
+    , m_labelsAngle(0)
+    , m_labelsPrecision(6)
+    , m_labelsDirty(true)
+    , m_barDelegateDirty(false)
 {
 }
 
@@ -1088,7 +1179,7 @@ void QBarSeriesPrivate::setBarWidth(qreal width)
     width = std::clamp<qreal>(width, 0.0, 1.0);
     if (!qFuzzyCompare(width, m_barWidth)) {
         m_barWidth = width;
-        q->update();
+        emit q->update();
     }
 }
 
@@ -1111,9 +1202,7 @@ void QBarSeriesPrivate::setVisible(bool visible)
 
 void QBarSeriesPrivate::setLabelsVisible(bool visible)
 {
-    Q_Q(QBarSeries);
     m_labelsVisible = visible;
-    emit q->labelsVisibleChanged(visible);
 }
 
 qreal QBarSeriesPrivate::min()
