@@ -10,17 +10,23 @@
 #include <private/qareaseries_p.h>
 #include <private/qgraphsview_p.h>
 #include <private/qxyseries_p.h>
+#include <QtQuick/private/qquicktaphandler_p.h>
 
 QT_BEGIN_NAMESPACE
 
-AreaRenderer::AreaRenderer(QGraphsView *graph)
+AreaRenderer::AreaRenderer(QGraphsView *graph, bool clipPlotArea)
     : QQuickItem(graph)
     , m_graph(graph)
 {
     setFlag(QQuickItem::ItemHasContents);
-    setClip(true);
+    setClip(clipPlotArea);
     m_shape.setParentItem(this);
     m_shape.setPreferredRendererType(QQuickShape::CurveRenderer);
+
+    m_tapHandler = new QQuickTapHandler(this);
+    connect(m_tapHandler, &QQuickTapHandler::singleTapped, this, &AreaRenderer::onSingleTapped);
+    connect(m_tapHandler, &QQuickTapHandler::doubleTapped, this, &AreaRenderer::onDoubleTapped);
+    connect(m_tapHandler, &QQuickTapHandler::pressedChanged, this, &AreaRenderer::onPressedChanged);
 }
 
 AreaRenderer::~AreaRenderer()
@@ -28,41 +34,60 @@ AreaRenderer::~AreaRenderer()
     qDeleteAll(m_groups);
 }
 
-void AreaRenderer::calculateRenderCoordinates(qreal origX,
-                                              qreal origY,
-                                              qreal *renderX,
-                                              qreal *renderY) const
+void AreaRenderer::resetShapePathCount()
 {
+    m_currentShapePathIndex = 0;
+}
+
+void AreaRenderer::calculateRenderCoordinates(
+    QAreaSeries *series, qreal origX, qreal origY, qreal *renderX, qreal *renderY) const
+{
+    auto &axY = m_graph->m_axisRenderer->getAxisY(series);
+
+    if (m_graph->orientation() != Qt::Vertical) {
+        std::swap(origX, origY);
+        origY = axY.maxValue - origY;
+    }
+
     *renderX = m_areaWidth * origX * m_maxHorizontal - m_horizontalOffset;
     *renderY = m_areaHeight - m_areaHeight * origY * m_maxVertical
                + m_verticalOffset;
 }
 
-void AreaRenderer::calculateAxisCoordinates(qreal origX,
-                                            qreal origY,
-                                            qreal *axisX,
-                                            qreal *axisY) const
+void AreaRenderer::calculateAxisCoordinates(
+    QAreaSeries *series, qreal origX, qreal origY, qreal *axisX, qreal *axisY) const
 {
-    *axisX = origX / m_areaWidth
-             / m_maxHorizontal;
-    *axisY = m_graph->m_axisRenderer->m_axisVerticalValueRange
-             - origY / m_areaHeight / m_maxVertical;
+    auto &axY = m_graph->m_axisRenderer->getAxisY(series);
+
+    if (m_graph->orientation() != Qt::Vertical) {
+        std::swap(origX, origY);
+        origY = axY.maxValue - origY;
+    }
+
+    *axisX = origX / m_areaWidth / m_maxHorizontal;
+    *axisY = axY.valueRange - origY / m_areaHeight / m_maxVertical;
 }
 
 void AreaRenderer::handlePolish(QAreaSeries *series)
 {
     auto theme = m_graph->theme();
-    if (!theme)
+    if (!theme) {
+        qCCritical(lcCritical2D, "theme not found.");
         return;
+    }
 
-    if (!m_graph->m_axisRenderer)
+    if (!m_graph->m_axisRenderer) {
+        qCCritical(lcCritical2D, "axis renderer not found.");
         return;
+    }
 
     QXYSeries *upper = series->upperSeries();
     QXYSeries *lower = series->lowerSeries();
 
-    if (!upper)
+    if (!upper) {
+        qCCritical(lcCritical2D, "upperSeries not found.");
         return;
+    }
 
     if (!m_groups.contains(series)) {
         PointGroup *group = new PointGroup();
@@ -76,82 +101,31 @@ void AreaRenderer::handlePolish(QAreaSeries *series)
 
     auto group = m_groups.value(series);
 
-    if (upper->points().count() < 2) {
-        auto pathElements = group->shapePath->pathElements();
-        pathElements.clear(&pathElements);
+    auto data = m_shape.data();
+    group->shapePath = qobject_cast<QQuickShapePath *>(data.at(&data, m_currentShapePathIndex));
 
-        for (auto path : group->paths)
-            path->deleteLater();
-
-        group->paths.clear();
-
-        return;
-    }
-
-    if (lower && lower->points().count() < 2) {
-        auto pathElements = group->shapePath->pathElements();
-        pathElements.clear(&pathElements);
-
-        for (auto path : group->paths)
-            path->deleteLater();
-
-        group->paths.clear();
+    m_currentShapePathIndex++;
 
+    if (upper->points().count() < 2 || (lower && lower->points().count() < 2)) {
+        auto painterPath = group->painterPath;
+        painterPath.clear();
+        group->shapePath->setPath(painterPath);
         return;
     }
 
     m_areaWidth = width();
     m_areaHeight = height();
 
-    m_maxVertical = m_graph->m_axisRenderer->m_axisVerticalValueRange > 0
-                        ? 1.0 / m_graph->m_axisRenderer->m_axisVerticalValueRange
-                        : 100.0;
-    m_maxHorizontal = m_graph->m_axisRenderer->m_axisHorizontalValueRange > 0
-                          ? 1.0 / m_graph->m_axisRenderer->m_axisHorizontalValueRange
-                          : 100.0;
-    m_verticalOffset = (m_graph->m_axisRenderer->m_axisVerticalMinValue
-                        / m_graph->m_axisRenderer->m_axisVerticalValueRange)
-                       * m_areaHeight;
-    m_horizontalOffset = (m_graph->m_axisRenderer->m_axisHorizontalMinValue
-                          / m_graph->m_axisRenderer->m_axisHorizontalValueRange)
-                         * m_areaWidth;
+    auto &axisX = m_graph->m_axisRenderer->getAxisX(group->series);
+    auto &axisY = m_graph->m_axisRenderer->getAxisY(group->series);
 
-    qsizetype pointCount = upper->points().size();
-    qsizetype currentSize = group->paths.size();
-    qsizetype extraCount = lower ? lower->points().size() : 2;
-    for (qsizetype i = currentSize; i < pointCount + extraCount; ++i) {
-        QQuickCurve *path = nullptr;
+    m_maxVertical = axisY.valueRange > 0 ? 1.0 / axisY.valueRange : 100.0;
+    m_maxHorizontal = axisX.valueRange > 0 ? 1.0 / axisX.valueRange : 100.0;
+    m_verticalOffset = (axisY.minValue / axisY.valueRange) * m_areaHeight;
+    m_horizontalOffset = (axisX.minValue / axisX.valueRange) * m_areaWidth;
 
-        if (!lower || (lower && i < pointCount - 1)) {
-            if (upper->type() == QAbstractSeries::SeriesType::Line || (!lower && i >= pointCount - 1))
-                path = new QQuickPathLine(group->shapePath);
-            else if (upper->type() == QAbstractSeries::SeriesType::Spline)
-                path = new QQuickPathCubic(group->shapePath);
-        } else if (lower && (i == pointCount - 1 || i == pointCount + extraCount - 1)) {
-            path = new QQuickPathLine(group->shapePath);
-        } else {
-            if (lower->type() == QAbstractSeries::SeriesType::Line)
-                path = new QQuickPathLine(group->shapePath);
-            else if (lower->type() == QAbstractSeries::SeriesType::Spline)
-                path = new QQuickPathCubic(group->shapePath);
-        }
-
-        if (path && series->isVisible()) {
-            auto pathElements = group->shapePath->pathElements();
-            pathElements.append(&pathElements, path);
-            group->paths << path;
-        }
-    }
-
-    auto pathElements = group->shapePath->pathElements();
-    if (!series->isVisible() && pathElements.count(&pathElements) > 0) {
-        pathElements.clear(&pathElements);
-
-        for (auto path : group->paths)
-            path->deleteLater();
-
-        group->paths.clear();
-    }
+    auto &painterPath = group->painterPath;
+    painterPath.clear();
 
     if (group->colorIndex < 0) {
         group->colorIndex = m_graph->graphSeriesCount();
@@ -186,96 +160,109 @@ void AreaRenderer::handlePolish(QAreaSeries *series)
 
     auto &&upperPoints = upper->points();
     QList<QPointF> fittedPoints;
+#ifdef USE_SPLINEGRAPH
     if (upper->type() == QAbstractSeries::SeriesType::Spline)
         fittedPoints = qobject_cast<QSplineSeries *>(upper)->getControlPoints();
+#endif
 
     int extraPointCount = lower ? 0 : 3;
 
     if (series->isVisible()) {
+        qreal prevUpperY = 0;
         for (int i = 0, j = 0; i < upperPoints.size() + extraPointCount; ++i, ++j) {
-            qreal x, y;
-            if (i == upperPoints.size())
-                calculateRenderCoordinates(upperPoints[upperPoints.size() - 1].x(), 0, &x, &y);
-            else if (i == upperPoints.size() + 1)
-                calculateRenderCoordinates(upperPoints[0].x(), 0, &x, &y);
-            else if (i == upperPoints.size() + 2)
-                calculateRenderCoordinates(upperPoints[0].x(), upperPoints[0].y(), &x, &y);
-            else
-                calculateRenderCoordinates(upperPoints[i].x(), upperPoints[i].y(), &x, &y);
+            qreal x;
+            qreal y;
+            qreal upperX;
+            qreal upperY;
+            if (i == upperPoints.size()) {
+                upperX = upperPoints[upperPoints.size() - 1].x();
+                upperY = 0;
+            } else if (i == upperPoints.size() + 1) {
+                upperX = upperPoints[0].x();
+                upperY = 0;
+            } else if (i == upperPoints.size() + 2) {
+                upperX = upperPoints[0].x();
+                upperY = upperPoints[0].y();
+            } else {
+                upperX = upperPoints[i].x();
+                upperY = upperPoints[i].y();
+            }
+            calculateRenderCoordinates(series, upperX, upperY, &x, &y);
 
             if (i == 0) {
-                group->shapePath->setStartX(x);
-                group->shapePath->setStartY(y);
+                painterPath.moveTo(x, y);
             } else {
-                group->paths[i - 1]->setX(x);
-                group->paths[i - 1]->setY(y);
-
-                auto *cubicPath = qobject_cast<QQuickPathCubic *>(group->paths[i - 1]);
-
-                if (cubicPath) {
+                if (i < upper->points().size()
+                    && upper->type() == QAbstractSeries::SeriesType::Spline) {
                     qreal x1, y1, x2, y2;
-                    calculateRenderCoordinates(fittedPoints[j - 1].x(),
+                    calculateRenderCoordinates(series,
+                                               fittedPoints[j - 1].x(),
                                                fittedPoints[j - 1].y(),
                                                &x1,
                                                &y1);
-                    calculateRenderCoordinates(fittedPoints[j].x(), fittedPoints[j].y(), &x2, &y2);
+                    calculateRenderCoordinates(series,
+                                               fittedPoints[j].x(),
+                                               fittedPoints[j].y(),
+                                               &x2,
+                                               &y2);
 
+                    painterPath.cubicTo(x1, y1, x2, y2, x, y);
                     ++j;
-
-                    cubicPath->setControl2X(x2);
-                    cubicPath->setControl1X(x1);
-                    cubicPath->setControl2Y(y2);
-                    cubicPath->setControl1Y(y1);
+                } else {
+                    painterPath.lineTo(x, y);
+                    if (i != 0 && i < upper->points().size()
+                    && upperY == 0 && prevUpperY == 0) {
+                        painterPath.moveTo(x, y);
+                    }
                 }
             }
+            prevUpperY = upperY;
         }
     }
 
     if (lower && series->isVisible()) {
         auto &&lowerPoints = lower->points();
         QList<QPointF> fittedPoints;
-        if (upper->type() == QAbstractSeries::SeriesType::Spline)
-            fittedPoints = qobject_cast<QSplineSeries *>(upper)->getControlPoints();
+#ifdef USE_SPLINEGRAPH
+        if (lower->type() == QAbstractSeries::SeriesType::Spline)
+            fittedPoints = qobject_cast<QSplineSeries *>(lower)->getControlPoints();
+#endif
 
         for (int i = 0, j = 0; i < lowerPoints.size(); ++i, ++j) {
             qreal x, y;
-            calculateRenderCoordinates(lowerPoints[lowerPoints.size() - 1 - i].x(),
+            calculateRenderCoordinates(series,
+                                       lowerPoints[lowerPoints.size() - 1 - i].x(),
                                        lowerPoints[lowerPoints.size() - 1 - i].y(),
                                        &x,
                                        &y);
 
-            group->paths[upperPoints.size() + i - 1]->setX(x);
-            group->paths[upperPoints.size() + i - 1]->setY(y);
-
-            auto *cubicPath = qobject_cast<QQuickPathCubic *>(
-                group->paths[upperPoints.size() + i - 1]);
-
-            if (cubicPath) {
+            if (i > 0 && lower->type() == QAbstractSeries::SeriesType::Spline) {
                 qreal x1, y1, x2, y2;
-                calculateRenderCoordinates(fittedPoints[fittedPoints.size() - 1 - j + 1].x(),
+                calculateRenderCoordinates(series,
+                                           fittedPoints[fittedPoints.size() - 1 - j + 1].x(),
                                            fittedPoints[fittedPoints.size() - 1 - j + 1].y(),
                                            &x1,
                                            &y1);
-                calculateRenderCoordinates(fittedPoints[fittedPoints.size() - 1 - j].x(),
+                calculateRenderCoordinates(series,
+                                           fittedPoints[fittedPoints.size() - 1 - j].x(),
                                            fittedPoints[fittedPoints.size() - 1 - j].y(),
                                            &x2,
                                            &y2);
 
+                painterPath.cubicTo(x1, y1, x2, y2, x, y);
                 ++j;
-
-                cubicPath->setControl2X(x2);
-                cubicPath->setControl1X(x1);
-                cubicPath->setControl2Y(y2);
-                cubicPath->setControl1Y(y1);
+            } else {
+                painterPath.lineTo(x, y);
             }
         }
 
         qreal x, y;
-        calculateRenderCoordinates(upperPoints[0].x(), upperPoints[0].y(), &x, &y);
-
-        group->paths[upperPoints.size() + lowerPoints.size() - 1]->setX(x);
-        group->paths[upperPoints.size() + lowerPoints.size() - 1]->setY(y);
+        calculateRenderCoordinates(series, upperPoints[0].x(), upperPoints[0].y(), &x, &y);
+        painterPath.lineTo(x, y);
     }
+
+    group->shapePath->setPath(painterPath);
+
     QList<QLegendData> legendDataList = {{color, borderColor, series->name()}};
     series->d_func()->setLegendData(legendDataList);
 }
@@ -287,10 +274,9 @@ void AreaRenderer::afterPolish(QList<QAbstractSeries *> &cleanupSeries)
         if (areaSeries && m_groups.contains(areaSeries)) {
             auto group = m_groups.value(areaSeries);
 
-            if (group->shapePath) {
-                auto pathElements = group->shapePath->pathElements();
-                pathElements.clear(&pathElements);
-            }
+            auto painterPath = group->painterPath;
+            painterPath.clear();
+            group->shapePath->setPath(painterPath);
 
             delete group;
             m_groups.remove(areaSeries);
@@ -344,8 +330,12 @@ bool AreaRenderer::pointInArea(QPoint pt, QAreaSeries *series) const
 
     for (int i = 0; i < firstPoints->size() - 1; ++i) {
         qreal x1, y1, x2, y2, x3, y3, x4, y4;
-        calculateRenderCoordinates((*firstPoints)[i].x(), (*firstPoints)[i].y(), &x1, &y1);
-        calculateRenderCoordinates((*firstPoints)[i + 1].x(), (*firstPoints)[i + 1].y(), &x2, &y2);
+        calculateRenderCoordinates(series, (*firstPoints)[i].x(), (*firstPoints)[i].y(), &x1, &y1);
+        calculateRenderCoordinates(series,
+                                   (*firstPoints)[i + 1].x(),
+                                   (*firstPoints)[i + 1].y(),
+                                   &x2,
+                                   &y2);
 
         bool needSecondTriangleTest = true;
         if (series->lowerSeries()) {
@@ -361,13 +351,15 @@ bool AreaRenderer::pointInArea(QPoint pt, QAreaSeries *series) const
             if (secondIndex >= secondPoints->size())
                 needSecondTriangleTest = false;
 
-            calculateRenderCoordinates((*secondPoints)[firstIndex].x(),
+            calculateRenderCoordinates(series,
+                                       (*secondPoints)[firstIndex].x(),
                                        (*secondPoints)[firstIndex].y(),
                                        &x3,
                                        &y3);
 
             if (needSecondTriangleTest) {
-                calculateRenderCoordinates((*secondPoints)[secondIndex].x(),
+                calculateRenderCoordinates(series,
+                                           (*secondPoints)[secondIndex].x(),
                                            (*secondPoints)[secondIndex].y(),
                                            &x4,
                                            &y4);
@@ -376,8 +368,8 @@ bool AreaRenderer::pointInArea(QPoint pt, QAreaSeries *series) const
                 y4 = 0.0;
             }
         } else {
-            calculateRenderCoordinates(upperPoints[i].x(), 0, &x3, &y3);
-            calculateRenderCoordinates(upperPoints[i + 1].x(), 0, &x4, &y4);
+            calculateRenderCoordinates(series, upperPoints[i].x(), 0, &x3, &y3);
+            calculateRenderCoordinates(series, upperPoints[i + 1].x(), 0, &x4, &y4);
         }
 
         QPoint point1(x1, y1);
@@ -394,27 +386,6 @@ bool AreaRenderer::pointInArea(QPoint pt, QAreaSeries *series) const
     return false;
 }
 
-bool AreaRenderer::handleMousePress(QMouseEvent *event)
-{
-    bool handled = false;
-    for (auto &&group : m_groups) {
-        if (!group->series->isSelectable() || !group->series->isVisible())
-            continue;
-
-        if (!group->series->upperSeries() || group->series->upperSeries()->count() < 2)
-            continue;
-
-        if (group->series->lowerSeries() && group->series->lowerSeries()->count() < 2)
-            continue;
-
-        if (pointInArea(event->pos(), group->series)) {
-            group->series->setSelected(!group->series->isSelected());
-            handled = true;
-        }
-    }
-    return handled;
-}
-
 bool AreaRenderer::handleHoverMove(QHoverEvent *event)
 {
     bool handled = false;
@@ -435,10 +406,11 @@ bool AreaRenderer::handleHoverMove(QHoverEvent *event)
         bool hovering = false;
         if (pointInArea(position.toPoint(), group->series)) {
             qreal x, y;
-            calculateAxisCoordinates(position.x(), position.y(), &x, &y);
+            calculateAxisCoordinates(group->series, position.x(), position.y(), &x, &y);
 
             if (!group->hover) {
                 group->hover = true;
+                group->series->setHovered(true);
                 emit group->series->hoverEnter(name, position, QPointF(x, y));
             }
 
@@ -449,6 +421,7 @@ bool AreaRenderer::handleHoverMove(QHoverEvent *event)
 
         if (!hovering && group->hover) {
             group->hover = false;
+            group->series->setHovered(false);
             emit group->series->hoverExit(name, position);
             handled = true;
         }
@@ -456,4 +429,85 @@ bool AreaRenderer::handleHoverMove(QHoverEvent *event)
     return handled;
 }
 
+void AreaRenderer::onSingleTapped(QEventPoint eventPoint, Qt::MouseButton button)
+{
+    Q_UNUSED(button)
+
+    for (auto &&group : m_groups) {
+        if (!group->series->isSelectable() || !group->series->isVisible())
+            continue;
+
+        if (!group->series->upperSeries() || group->series->upperSeries()->count() < 2)
+            continue;
+
+        if (group->series->lowerSeries() && group->series->lowerSeries()->count() < 2)
+            continue;
+
+        if (pointInArea(eventPoint.position().toPoint(), group->series)) {
+            group->series->setSelected(!group->series->isSelected());
+            m_graph->polishAndUpdate();
+            qreal x;
+            qreal y;
+            calculateAxisCoordinates(group->series,
+                                     eventPoint.position().x(),
+                                     eventPoint.position().y(),
+                                     &x,
+                                     &y);
+            emit group->series->clicked(QPoint(x, y));
+        }
+    }
+}
+
+void AreaRenderer::onDoubleTapped(QEventPoint eventPoint, Qt::MouseButton button)
+{
+    Q_UNUSED(button)
+
+    for (auto &&group : m_groups) {
+        if (!group->series->isSelectable() || !group->series->isVisible())
+            continue;
+
+        if (!group->series->upperSeries() || group->series->upperSeries()->count() < 2)
+            continue;
+
+        if (group->series->lowerSeries() && group->series->lowerSeries()->count() < 2)
+            continue;
+
+        if (pointInArea(eventPoint.position().toPoint(), group->series)) {
+            qreal x;
+            qreal y;
+            calculateAxisCoordinates(group->series,
+                                     eventPoint.position().x(),
+                                     eventPoint.position().y(),
+                                     &x,
+                                     &y);
+            emit group->series->doubleClicked(QPoint(x, y));
+        }
+    }
+}
+
+void AreaRenderer::onPressedChanged()
+{
+    QPointF position = m_tapHandler->point().position();
+    for (auto &&group : m_groups) {
+        if (!group->series->isSelectable() || !group->series->isVisible())
+            continue;
+
+        if (!group->series->upperSeries() || group->series->upperSeries()->count() < 2)
+            continue;
+
+        if (group->series->lowerSeries() && group->series->lowerSeries()->count() < 2)
+            continue;
+
+        if (pointInArea(position.toPoint(), group->series)) {
+            qreal x;
+            qreal y;
+            calculateAxisCoordinates(group->series, position.x(), position.y(), &x, &y);
+            if (m_tapHandler->isPressed())
+                emit group->series->pressed(QPoint(x, y));
+            else
+                emit group->series->released(QPoint(x, y));
+        }
+    }
+}
+
 QT_END_NAMESPACE
