@@ -5,6 +5,7 @@
 #include "qabstractdataproxy_p.h"
 #include "qquickgraphsitem_p.h"
 #include "utils_p.h"
+#include "qgraphs3dlogging_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -32,13 +33,14 @@ QT_BEGIN_NAMESPACE
 
 /*!
  * \qmltype Abstract3DSeries
+ * \qmlabstract
  * \inqmlmodule QtGraphs
  * \ingroup graphs_qml_3D
- * \instantiates QAbstract3DSeries
+ * \nativetype QAbstract3DSeries
  * \brief A base type for all 3D data series.
  *
- * This type is uncreatable, but contains properties that are exposed via the
- * following subtypes: Bar3DSeries, Scatter3DSeries, and Surface3DSeries.
+ * This abstract class serves as a base class for the following subtypes:
+ * Bar3DSeries, Scatter3DSeries, and Surface3DSeries.
  *
  * For more information, see \l{Qt Graphs Data Handling with 3D}.
  */
@@ -51,13 +53,14 @@ QT_BEGIN_NAMESPACE
  * \value None
  *        No series type.
  * \value Bar
- *        Series type for Q3DBars.
+ *        Series type for Q3DBarsWidgetItem.
  * \value Scatter
- *        Series type for Q3DScatter.
+ *        Series type for Q3DScatterWidgetItem.
  * \value Surface
- *        Series type for Q3DSurface.
+ *        Series type for Q3DSurfaceWidgetItem.
  */
 
+
 /*!
  *  \enum QAbstract3DSeries::Mesh
  *
@@ -82,15 +85,26 @@ QT_BEGIN_NAMESPACE
  *  \value Sphere
  *         Sphere.
  *  \value Minimal
- *         The minimal 3D mesh: a triangular pyramid. Usable only with Q3DScatter.
+ *         The minimal 3D mesh: a triangular pyramid. Usable only with Q3DScatterWidgetItem.
  *  \value Arrow
  *         Arrow pointing upwards.
  *  \value Point
- *         2D point. Usable only with Q3DScatter.
- *         Shadows do not affect this style. Color style Q3DTheme::ColorStyle::ObjectGradient
+ *         2D point. Usable only with Q3DScatterWidgetItem.
+ *         Shadows do not affect this style. Color style QGraphsTheme::ColorStyle::ObjectGradient
  *         is not supported by this style.
  */
 
+/*!
+ *  \enum QAbstract3DSeries::LightingMode
+ *
+ *  Predefined lighting modes
+ *
+ *  \value Shaded
+ *      Graphs respond to real time lighting
+ *  \value Unshaded
+ *      Graphs do not respond to real time lighting
+ */
+
 /*!
  * \qmlproperty Abstract3DSeries.SeriesType Abstract3DSeries::type
  * The type of the series. One of the QAbstract3DSeries::SeriesType values.
@@ -102,19 +116,20 @@ QT_BEGIN_NAMESPACE
  *
  * The label format for data items in this series. This format is used for
  * single item labels, for example, when an item is selected. How the format is
- * interpreted depends on series type: Bar3DSeries, Scatter3DSeries,
- * Surface3DSeries.
+ * interpreted depends on series type.
+ *
+ * \sa Bar3DSeries, Scatter3DSeries, Surface3DSeries.
  */
 
 /*!
  * \qmlproperty bool Abstract3DSeries::visible
- * Sets the visibility of the series. If \c false, the series is not rendered.
+ * The visibility of the series. If \c false, the series is not rendered.
  */
 
 /*!
  * \qmlproperty Abstract3DSeries.Mesh Abstract3DSeries::mesh
  *
- * Sets the mesh of the items in the series, or the selection pointer in case of
+ * The mesh of the items in the series, or the selection pointer in case of
  * Surface3DSeries. If the mesh is
  * \l{QAbstract3DSeries::Mesh::UserDefined}{Abstract3DSeries.Mesh.UserDefined},
  * then the userDefinedMesh property must also be set for items to render
@@ -136,7 +151,7 @@ QT_BEGIN_NAMESPACE
 /*!
  * \qmlproperty quaternion Abstract3DSeries::meshRotation
  *
- * Sets the mesh rotation that is applied to all items of the series.
+ * The mesh rotation that is applied to all items of the series.
  * The rotation should be a normalized quaternion.
  * For those series types that support item specific rotation, the rotations are
  * multiplied together.
@@ -148,69 +163,79 @@ QT_BEGIN_NAMESPACE
 /*!
  * \qmlproperty string Abstract3DSeries::userDefinedMesh
  *
- * Sets the filename for a user defined custom mesh for objects that is used
+ * The filename for a user defined custom mesh for objects that is used
  * when \l mesh is
  * \l{QAbstract3DSeries::Mesh::UserDefined}{Abstract3DSeries.Mesh.UserDefined}.
- * \note The file needs to be in the Wavefront OBJ format and include
- * vertices, normals, and UVs. It also needs to be in triangles.
+ * \note The file needs to be in the QtQuick3D mesh format. Use the \c balsam
+ * conversion tool to create a mesh from other 3D model formats.
  */
 
 /*!
- * \qmlproperty Theme3D.ColorStyle Abstract3DSeries::colorStyle
+ * \qmlproperty GraphsTheme.ColorStyle Abstract3DSeries::colorStyle
  *
- * Sets the color style for the series.
+ * The color style for the series.
  *
- * \sa {Theme3D::colorStyle}{Theme3D.colorStyle}
+ * \sa {QGraphsTheme::ColorStyle}{GraphsTheme.ColorStyle}
  */
 
 /*!
  * \qmlproperty color Abstract3DSeries::baseColor
  *
- * Sets the base color of the series.
+ * The base color of the series.
  *
- * \sa colorStyle, {Theme3D::baseColors}{Theme3D.baseColors}
+ * \sa colorStyle, {GraphsTheme::seriesColors}{GraphsTheme.seriesColors}
  */
 
 /*!
  * \qmlproperty Gradient Abstract3DSeries::baseGradient
  *
- * Sets the base gradient of the series.
+ * The base gradient of the series.
  *
- * \sa colorStyle, {Theme3D::baseGradients}{Theme3D.baseGradients}
+ * \sa colorStyle
  */
 
 /*!
  * \qmlproperty color Abstract3DSeries::singleHighlightColor
  *
- * Sets the single item highlight color of the series.
+ * The single item highlight color of the series.
  *
- * \sa colorStyle, {Theme3D::singleHighlightColor}{Theme3D.singleHighlightColor}
+ * \sa colorStyle, {GraphsTheme::singleHighlightColor}{GraphsTheme.singleHighlightColor}
  */
 
 /*!
  * \qmlproperty Gradient Abstract3DSeries::singleHighlightGradient
  *
- * Sets the single item highlight gradient of the series.
+ * The single item highlight gradient of the series.
  *
  * \sa colorStyle,
- * {Theme3D::singleHighlightGradient}{Theme3D.singleHighlightGradient}
+ * {GraphsTheme::singleHighlightGradient}{GraphsTheme.singleHighlightGradient}
  */
 
 /*!
  * \qmlproperty color Abstract3DSeries::multiHighlightColor
  *
- * Sets the multiple item highlight color of the series.
+ * The multiple item highlight color of the series.
  *
- * \sa colorStyle, {Theme3D::multiHighlightColor}{Theme3D.multiHighlightColor}
+ * \sa colorStyle, {GraphsTheme::multiHighlightColor}{GraphsTheme.multiHighlightColor}
  */
 
 /*!
  * \qmlproperty Gradient Abstract3DSeries::multiHighlightGradient
  *
- * Sets the multiple item highlight gradient of the series.
+ * The multiple item highlight gradient of the series.
  *
  * \sa colorStyle,
- * {Theme3D::multiHighlightGradient}{Theme3D.multiHighlightGradient}
+ * {GraphsTheme::multiHighlightGradient}{GraphsTheme.multiHighlightGradient}
+ */
+
+/*!
+ * \qmlproperty Abstract3DSeries.LightingMode Abstract3DSeries::lightingMode
+ * \since 6.10
+ *
+ * The lighting mode of the items in the series.
+ * The default value is \l{QAbstract3DSeries::LightingMode::Shaded}
+ *
+ * \sa QAbstract3DSeries::LightingMode
  */
 
 /*!
@@ -251,6 +276,93 @@ QT_BEGIN_NAMESPACE
  * \sa meshRotation
  */
 
+/*!
+    \qmlsignal Abstract3DSeries::itemLabelFormatChanged(string format)
+
+    This signal is emitted when itemLabelFormat changes to \a format.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::visibilityChanged(bool visible)
+
+    This signal is emitted when the series visibility changes to \a visible.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::meshChanged(Abstract3DSeries.Mesh mesh)
+
+    This signal is emitted when \l mesh changes to \a mesh.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::meshSmoothChanged(bool enabled)
+
+    This signal is emitted when meshSmooth changes to \a enabled.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::meshRotationChanged(quaternion rotation)
+
+    This signal is emitted when meshRotation changes to \a rotation.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::userDefinedMeshChanged(string fileName)
+
+    This signal is emitted when userDefinedMesh changes to \a fileName.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::colorStyleChanged(GraphsTheme.ColorStyle style)
+
+    This signal is emitted when colorStyle changes to \a style.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::baseColorChanged(color color)
+
+    This signal is emitted when baseColor changes to \a color.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::baseGradientChanged(Gradient gradient)
+
+    This signal is emitted when baseGradient changes to \a gradient.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::singleHighlightColorChanged(color color)
+
+    This signal is emitted when singleHighlightColor changes to \a color.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::singleHighlightGradientChanged(Gradient gradient)
+
+    This signal is emitted when singleHighlightGradient changes to \a gradient.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::multiHighlightColorChanged(color color)
+
+    This signal is emitted when multiHighlightColor changes to \a color.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::multiHighlightGradientChanged(Gradient gradient)
+
+    This signal is emitted when multiHighlightGradient changes to \a gradient.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::lightingModeChanged(Abstract3DSeries.LightingMode lightingMode)
+
+    This signal is emitted when \l lightingMode changes to \a lightingMode.
+    \since 6.10
+*/
+/*!
+    \qmlsignal Abstract3DSeries::nameChanged(string name)
+
+    This signal is emitted when \l name changes to \a name.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::itemLabelChanged(string label)
+
+    This signal is emitted when itemLabel changes to \a label.
+*/
+/*!
+    \qmlsignal Abstract3DSeries::itemLabelVisibilityChanged(bool visible)
+
+    This signal is emitted when itemLabelVisibility changes to \a visible.
+*/
+
 /*!
  * \internal
  */
@@ -270,7 +382,7 @@ QAbstract3DSeries::~QAbstract3DSeries() {}
  */
 QAbstract3DSeries::SeriesType QAbstract3DSeries::type() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_type;
 }
 
@@ -281,20 +393,26 @@ QAbstract3DSeries::SeriesType QAbstract3DSeries::type() const
  *
  * This format is used for single item labels,
  * for example, when an item is selected. How the format is interpreted depends
- * on series type: QBar3DSeries, QScatter3DSeries, QSurface3DSeries.
+ * on series type.
+ *
+ * \sa QBar3DSeries, QScatter3DSeries, QSurface3DSeries.
  */
 void QAbstract3DSeries::setItemLabelFormat(const QString &format)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_itemLabelFormat != format) {
-        d->setItemLabelFormat(format);
-        emit itemLabelFormatChanged(format);
+    if (d->m_itemLabelFormat == format) {
+        qCDebug(lcProperties3D, "%s value: %s is same than it already was.",
+                qUtf8Printable(QLatin1String(__FUNCTION__)), qUtf8Printable(format));
+        return;
     }
+
+    d->setItemLabelFormat(format);
+    emit itemLabelFormatChanged(format);
 }
 
 QString QAbstract3DSeries::itemLabelFormat() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_itemLabelFormat;
 }
 
@@ -309,15 +427,19 @@ QString QAbstract3DSeries::itemLabelFormat() const
 void QAbstract3DSeries::setVisible(bool visible)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_visible != visible) {
-        d->setVisible(visible);
-        emit visibilityChanged(visible);
+    if (d->m_visible == visible) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << visible;
+        return;
     }
+
+    d->setVisible(visible);
+    emit visibleChanged(visible);
 }
 
 bool QAbstract3DSeries::isVisible() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_visible;
 }
 
@@ -338,16 +460,22 @@ void QAbstract3DSeries::setMesh(QAbstract3DSeries::Mesh mesh)
     if ((mesh == QAbstract3DSeries::Mesh::Point || mesh == QAbstract3DSeries::Mesh::Minimal
          || mesh == QAbstract3DSeries::Mesh::Arrow)
         && type() != QAbstract3DSeries::SeriesType::Scatter) {
-        qWarning() << "Specified style is only supported for QScatter3DSeries.";
-    } else if (d->m_mesh != mesh) {
-        d->setMesh(mesh);
-        emit meshChanged(mesh);
+        qCWarning(lcProperties3D, "%s specified style is only supported for QScatter3DSeries.",
+                  qUtf8Printable(QLatin1String(__FUNCTION__)));
+        return;
+    } else if (d->m_mesh == mesh) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << mesh;
+        return;
     }
+
+    d->setMesh(mesh);
+    emit meshChanged(mesh);
 }
 
 QAbstract3DSeries::Mesh QAbstract3DSeries::mesh() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_mesh;
 }
 
@@ -363,15 +491,19 @@ QAbstract3DSeries::Mesh QAbstract3DSeries::mesh() const
 void QAbstract3DSeries::setMeshSmooth(bool enable)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_meshSmooth != enable) {
-        d->setMeshSmooth(enable);
-        emit meshSmoothChanged(enable);
+    if (d->m_meshSmooth == enable) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << enable;
+        return;
     }
+
+    d->setMeshSmooth(enable);
+    emit meshSmoothChanged(enable);
 }
 
 bool QAbstract3DSeries::isMeshSmooth() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_meshSmooth;
 }
 
@@ -390,15 +522,19 @@ bool QAbstract3DSeries::isMeshSmooth() const
 void QAbstract3DSeries::setMeshRotation(const QQuaternion &rotation)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_meshRotation != rotation) {
-        d->setMeshRotation(rotation);
-        emit meshRotationChanged(rotation);
+    if (d->m_meshRotation == rotation) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << rotation;
+        return;
     }
+
+    d->setMeshRotation(rotation);
+    emit meshRotationChanged(rotation);
 }
 
 QQuaternion QAbstract3DSeries::meshRotation() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_meshRotation;
 }
 
@@ -408,7 +544,7 @@ QQuaternion QAbstract3DSeries::meshRotation() const
  *
  * \sa meshRotation
  */
-void QAbstract3DSeries::setMeshAxisAndAngle(const QVector3D &axis, float angle)
+void QAbstract3DSeries::setMeshAxisAndAngle(QVector3D axis, float angle)
 {
     setMeshRotation(QQuaternion::fromAxisAndAngle(axis, angle));
 }
@@ -419,21 +555,25 @@ void QAbstract3DSeries::setMeshAxisAndAngle(const QVector3D &axis, float angle)
  * \brief The filename for a user defined custom mesh for objects.
  *
  * The custom mesh is used when \l mesh is MeshUserDefined.
- * \note The file needs to be in the Wavefront OBJ format and include
- * vertices, normals, and UVs. It also needs to be in triangles.
+ * \note The file needs to be in the QtQuick3D mesh format. Use the \c balsam
+ * conversion tool to create a mesh from other 3D model formats.
  */
 void QAbstract3DSeries::setUserDefinedMesh(const QString &fileName)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_userDefinedMesh != fileName) {
-        d->setUserDefinedMesh(fileName);
-        emit userDefinedMeshChanged(fileName);
+    if (d->m_userDefinedMesh == fileName) {
+        qCDebug(lcProperties3D, "%s value %s is same than what is already being used.",
+                qUtf8Printable(QLatin1String(__FUNCTION__)), qUtf8Printable(fileName));
+        return;
     }
+
+    d->setUserDefinedMesh(fileName);
+    emit userDefinedMeshChanged(fileName);
 }
 
 QString QAbstract3DSeries::userDefinedMesh() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_userDefinedMesh;
 }
 
@@ -442,21 +582,27 @@ QString QAbstract3DSeries::userDefinedMesh() const
  *
  * \brief The color style for the series.
  *
- * \sa Q3DTheme::ColorStyle
+ * \sa QGraphsTheme::ColorStyle
  */
-void QAbstract3DSeries::setColorStyle(Q3DTheme::ColorStyle style)
+void QAbstract3DSeries::setColorStyle(QGraphsTheme::ColorStyle style)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_colorStyle != style) {
-        d->setColorStyle(style);
-        emit colorStyleChanged(style);
+    if (d->m_colorStyle == style) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << style;
+        d->m_themeTracker.colorStyleOverride = true;
+        return;
     }
+
+    d->setColorStyle(style);
+    emit colorStyleChanged(style);
+
     d->m_themeTracker.colorStyleOverride = true;
 }
 
-Q3DTheme::ColorStyle QAbstract3DSeries::colorStyle() const
+QGraphsTheme::ColorStyle QAbstract3DSeries::colorStyle() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_colorStyle;
 }
 
@@ -465,21 +611,27 @@ Q3DTheme::ColorStyle QAbstract3DSeries::colorStyle() const
  *
  * \brief The base color of the series.
  *
- * \sa colorStyle, Q3DTheme::baseColors
+ * \sa colorStyle, QGraphsTheme::seriesColors
  */
-void QAbstract3DSeries::setBaseColor(const QColor &color)
+void QAbstract3DSeries::setBaseColor(QColor color)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_baseColor != color) {
-        d->setBaseColor(color);
-        emit baseColorChanged(color);
+    if (d->m_baseColor == color) {
+        qCDebug(lcProperties3D, "%s value is already set to: %s",
+                qUtf8Printable(QLatin1String(__FUNCTION__)), qUtf8Printable(color.name()));
+        d->m_themeTracker.baseColorOverride = true;
+        return;
     }
+
+    d->setBaseColor(color);
+    emit baseColorChanged(color);
+
     d->m_themeTracker.baseColorOverride = true;
 }
 
 QColor QAbstract3DSeries::baseColor() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_baseColor;
 }
 
@@ -488,21 +640,26 @@ QColor QAbstract3DSeries::baseColor() const
  *
  * \brief The base gradient of the series.
  *
- * \sa colorStyle, Q3DTheme::baseGradients
+ * \sa colorStyle, QGraphsTheme::seriesGradients
  */
 void QAbstract3DSeries::setBaseGradient(const QLinearGradient &gradient)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_baseGradient != gradient) {
-        d->setBaseGradient(gradient);
-        emit baseGradientChanged(gradient);
+    if (d->m_baseGradient == gradient) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << gradient;
+        d->m_themeTracker.baseGradientOverride = true;
+        return;
     }
+    d->setBaseGradient(gradient);
+    emit baseGradientChanged(gradient);
+
     d->m_themeTracker.baseGradientOverride = true;
 }
 
 QLinearGradient QAbstract3DSeries::baseGradient() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_baseGradient;
 }
 
@@ -511,21 +668,26 @@ QLinearGradient QAbstract3DSeries::baseGradient() const
  *
  * \brief The single item highlight color of the series.
  *
- * \sa colorStyle, Q3DTheme::singleHighlightColor
+ * \sa colorStyle, QGraphsTheme::singleHighlightColor
  */
-void QAbstract3DSeries::setSingleHighlightColor(const QColor &color)
+void QAbstract3DSeries::setSingleHighlightColor(QColor color)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_singleHighlightColor != color) {
-        d->setSingleHighlightColor(color);
-        emit singleHighlightColorChanged(color);
+    if (d->m_singleHighlightColor == color) {
+        qCDebug(lcProperties3D, "%s value is already set to: %s",
+                qUtf8Printable(QLatin1String(__FUNCTION__)), qUtf8Printable(color.name()));
+        d->m_themeTracker.singleHighlightColorOverride = true;
+        return;
     }
+
+    d->setSingleHighlightColor(color);
+    emit singleHighlightColorChanged(color);
     d->m_themeTracker.singleHighlightColorOverride = true;
 }
 
 QColor QAbstract3DSeries::singleHighlightColor() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_singleHighlightColor;
 }
 
@@ -534,21 +696,26 @@ QColor QAbstract3DSeries::singleHighlightColor() const
  *
  * \brief The single item highlight gradient of the series.
  *
- * \sa colorStyle, Q3DTheme::singleHighlightGradient
+ * \sa colorStyle, QGraphsTheme::singleHighlightGradient
  */
 void QAbstract3DSeries::setSingleHighlightGradient(const QLinearGradient &gradient)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_singleHighlightGradient != gradient) {
-        d->setSingleHighlightGradient(gradient);
-        emit singleHighlightGradientChanged(gradient);
+    if (d->m_singleHighlightGradient == gradient) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << gradient;
+        d->m_themeTracker.singleHighlightGradientOverride = true;
+        return;
     }
+
+    d->setSingleHighlightGradient(gradient);
+    emit singleHighlightGradientChanged(gradient);
     d->m_themeTracker.singleHighlightGradientOverride = true;
 }
 
 QLinearGradient QAbstract3DSeries::singleHighlightGradient() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_singleHighlightGradient;
 }
 
@@ -557,21 +724,26 @@ QLinearGradient QAbstract3DSeries::singleHighlightGradient() const
  *
  * \brief The multiple item highlight color of the series.
  *
- * \sa colorStyle, Q3DTheme::multiHighlightColor
+ * \sa colorStyle, QGraphsTheme::multiHighlightColor
  */
-void QAbstract3DSeries::setMultiHighlightColor(const QColor &color)
+void QAbstract3DSeries::setMultiHighlightColor(QColor color)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_multiHighlightColor != color) {
-        d->setMultiHighlightColor(color);
-        emit multiHighlightColorChanged(color);
+    if (d->m_multiHighlightColor == color) {
+        qCDebug(lcProperties3D, "%s value is already set to: %s",
+                qUtf8Printable(QLatin1String(__FUNCTION__)), qUtf8Printable(color.name()));
+        d->m_themeTracker.multiHighlightColorOverride = true;
+        return;
     }
+
+    d->setMultiHighlightColor(color);
+    emit multiHighlightColorChanged(color);
     d->m_themeTracker.multiHighlightColorOverride = true;
 }
 
 QColor QAbstract3DSeries::multiHighlightColor() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_multiHighlightColor;
 }
 
@@ -580,24 +752,51 @@ QColor QAbstract3DSeries::multiHighlightColor() const
  *
  * \brief The multiple item highlight gradient of the series.
  *
- * \sa colorStyle, Q3DTheme::multiHighlightGradient
+ * \sa colorStyle, QGraphsTheme::multiHighlightGradient
  */
 void QAbstract3DSeries::setMultiHighlightGradient(const QLinearGradient &gradient)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_multiHighlightGradient != gradient) {
-        d->setMultiHighlightGradient(gradient);
-        emit multiHighlightGradientChanged(gradient);
+    if (d->m_multiHighlightGradient == gradient) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << gradient;
+        d->m_themeTracker.multiHighlightGradientOverride = true;
+        return;
     }
+
+    d->setMultiHighlightGradient(gradient);
+    emit multiHighlightGradientChanged(gradient);
     d->m_themeTracker.multiHighlightGradientOverride = true;
 }
 
 QLinearGradient QAbstract3DSeries::multiHighlightGradient() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_multiHighlightGradient;
 }
 
+/*!
+ * \property QAbstract3DSeries::lightingMode
+ *
+ * \brief The lighting mode of the series
+ * \since 6.10
+ * \sa LightingMode
+ */
+void QAbstract3DSeries::setLightingMode(LightingMode lightingMode)
+{
+    Q_D(QAbstract3DSeries);
+    if (d->m_lightingMode != lightingMode) {
+        d->setLightingMode(lightingMode);
+        emit lightingModeChanged(lightingMode);
+    }
+}
+
+QAbstract3DSeries::LightingMode QAbstract3DSeries::lightingMode() const
+{
+    Q_D(const QAbstract3DSeries);
+    return d->m_lightingMode;
+}
+
 /*!
  * \property QAbstract3DSeries::name
  *
@@ -611,15 +810,19 @@ QLinearGradient QAbstract3DSeries::multiHighlightGradient() const
 void QAbstract3DSeries::setName(const QString &name)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_name != name) {
-        d->setName(name);
-        emit nameChanged(name);
+    if (d->m_name == name) {
+        qCDebug(lcProperties3D, "%s value is already set to: %s",
+                qUtf8Printable(QLatin1String(__FUNCTION__)), qUtf8Printable(name));
+        return;
     }
+
+    d->setName(name);
+    emit nameChanged(name);
 }
 
 QString QAbstract3DSeries::name() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_name;
 }
 
@@ -653,15 +856,19 @@ QString QAbstract3DSeries::itemLabel()
 void QAbstract3DSeries::setItemLabelVisible(bool visible)
 {
     Q_D(QAbstract3DSeries);
-    if (d->m_itemLabelVisible != visible) {
-        d->setItemLabelVisible(visible);
-        emit itemLabelVisibilityChanged(visible);
+    if (d->m_itemLabelVisible == visible) {
+        qCDebug(lcProperties3D) << __FUNCTION__
+            << "value is already set to:" << visible;
+        return;
     }
+
+    d->setItemLabelVisible(visible);
+    emit itemLabelVisibleChanged(visible);
 }
 
 bool QAbstract3DSeries::isItemLabelVisible() const
 {
-    const Q_D(QAbstract3DSeries);
+    Q_D(const QAbstract3DSeries);
     return d->m_itemLabelVisible;
 }
 
@@ -674,12 +881,13 @@ QAbstract3DSeriesPrivate::QAbstract3DSeriesPrivate(QAbstract3DSeries::SeriesType
     , m_graph(0)
     , m_mesh(QAbstract3DSeries::Mesh::Cube)
     , m_meshSmooth(false)
-    , m_colorStyle(Q3DTheme::ColorStyle::Uniform)
+    , m_colorStyle(QGraphsTheme::ColorStyle::Uniform)
     , m_baseColor(Qt::black)
     , m_singleHighlightColor(Qt::black)
     , m_multiHighlightColor(Qt::black)
     , m_itemLabelDirty(true)
     , m_itemLabelVisible(true)
+    , m_lightingMode(QAbstract3DSeries::LightingMode::Shaded)
 {}
 
 QAbstract3DSeriesPrivate::~QAbstract3DSeriesPrivate() {}
@@ -708,9 +916,9 @@ void QAbstract3DSeriesPrivate::setDataProxy(QAbstractDataProxy *proxy)
 void QAbstract3DSeriesPrivate::setGraph(QQuickGraphsItem *graph)
 {
     Q_Q(QAbstract3DSeries);
-    connectGraphAndProxy(graph);
     m_graph = graph;
     q->setParent(graph);
+    connectGraphAndProxy(graph);
     markItemLabelDirty();
 }
 
@@ -733,7 +941,7 @@ void QAbstract3DSeriesPrivate::setMesh(QAbstract3DSeries::Mesh mesh)
     if (m_graph) {
         m_graph->markSeriesVisualsDirty();
 
-        if (m_graph->optimizationHint() == QAbstract3DGraph::OptimizationHint::Default)
+        if (m_graph->optimizationHint() == QtGraphs3D::OptimizationHint::Default)
             m_graph->markDataDirty();
     }
 }
@@ -745,7 +953,7 @@ void QAbstract3DSeriesPrivate::setMeshSmooth(bool enable)
     if (m_graph) {
         m_graph->markSeriesVisualsDirty();
 
-        if (m_graph->optimizationHint() == QAbstract3DGraph::OptimizationHint::Default)
+        if (m_graph->optimizationHint() == QtGraphs3D::OptimizationHint::Default)
             m_graph->markDataDirty();
     }
 }
@@ -757,7 +965,7 @@ void QAbstract3DSeriesPrivate::setMeshRotation(const QQuaternion &rotation)
     if (m_graph) {
         m_graph->markSeriesVisualsDirty();
 
-        if (m_graph->optimizationHint() == QAbstract3DGraph::OptimizationHint::Default)
+        if (m_graph->optimizationHint() == QtGraphs3D::OptimizationHint::Default)
             m_graph->markDataDirty();
     }
 }
@@ -769,12 +977,12 @@ void QAbstract3DSeriesPrivate::setUserDefinedMesh(const QString &meshFile)
     if (m_graph) {
         m_graph->markSeriesVisualsDirty();
 
-        if (m_graph->optimizationHint() == QAbstract3DGraph::OptimizationHint::Default)
+        if (m_graph->optimizationHint() == QtGraphs3D::OptimizationHint::Default)
             m_graph->markDataDirty();
     }
 }
 
-void QAbstract3DSeriesPrivate::setColorStyle(Q3DTheme::ColorStyle style)
+void QAbstract3DSeriesPrivate::setColorStyle(QGraphsTheme::ColorStyle style)
 {
     m_colorStyle = style;
     m_changeTracker.colorStyleChanged = true;
@@ -782,7 +990,7 @@ void QAbstract3DSeriesPrivate::setColorStyle(Q3DTheme::ColorStyle style)
         m_graph->markSeriesVisualsDirty();
 }
 
-void QAbstract3DSeriesPrivate::setBaseColor(const QColor &color)
+void QAbstract3DSeriesPrivate::setBaseColor(QColor color)
 {
     m_baseColor = color;
     m_changeTracker.baseColorChanged = true;
@@ -799,7 +1007,7 @@ void QAbstract3DSeriesPrivate::setBaseGradient(const QLinearGradient &gradient)
         m_graph->markSeriesVisualsDirty();
 }
 
-void QAbstract3DSeriesPrivate::setSingleHighlightColor(const QColor &color)
+void QAbstract3DSeriesPrivate::setSingleHighlightColor(QColor color)
 {
     m_singleHighlightColor = color;
     m_changeTracker.singleHighlightColorChanged = true;
@@ -816,7 +1024,7 @@ void QAbstract3DSeriesPrivate::setSingleHighlightGradient(const QLinearGradient
         m_graph->markSeriesVisualsDirty();
 }
 
-void QAbstract3DSeriesPrivate::setMultiHighlightColor(const QColor &color)
+void QAbstract3DSeriesPrivate::setMultiHighlightColor(QColor color)
 {
     m_multiHighlightColor = color;
     m_changeTracker.multiHighlightColorChanged = true;
@@ -833,6 +1041,14 @@ void QAbstract3DSeriesPrivate::setMultiHighlightGradient(const QLinearGradient &
         m_graph->markSeriesVisualsDirty();
 }
 
+void QAbstract3DSeriesPrivate::setLightingMode(QAbstract3DSeries::LightingMode lightingMode)
+{
+    m_lightingMode = lightingMode;
+    if (m_graph)
+        m_graph->markSeriesVisualsDirty();
+    m_changeTracker.lightingModeChanged = true;
+}
+
 void QAbstract3DSeriesPrivate::setName(const QString &name)
 {
     m_name = name;
@@ -840,24 +1056,24 @@ void QAbstract3DSeriesPrivate::setName(const QString &name)
     m_changeTracker.nameChanged = true;
 }
 
-void QAbstract3DSeriesPrivate::resetToTheme(const Q3DTheme &theme, int seriesIndex, bool force)
+void QAbstract3DSeriesPrivate::resetToTheme(const QGraphsTheme &theme, qsizetype seriesIndex, bool force)
 {
     Q_Q(QAbstract3DSeries);
-    int themeIndex = seriesIndex;
+    qsizetype themeIndex = seriesIndex;
     if (force || !m_themeTracker.colorStyleOverride) {
         q->setColorStyle(theme.colorStyle());
         m_themeTracker.colorStyleOverride = false;
     }
-    if (force || !m_themeTracker.baseColorOverride) {
-        if (theme.baseColors().size() <= seriesIndex)
-            themeIndex = seriesIndex % theme.baseColors().size();
-        q->setBaseColor(theme.baseColors().at(themeIndex));
+    if (theme.seriesColors().size() && (force || !m_themeTracker.baseColorOverride)) {
+        if (theme.seriesColors().size() <= seriesIndex)
+            themeIndex = seriesIndex % theme.seriesColors().size();
+        q->setBaseColor(theme.seriesColors().at(themeIndex));
         m_themeTracker.baseColorOverride = false;
     }
-    if (force || !m_themeTracker.baseGradientOverride) {
-        if (theme.baseGradients().size() <= seriesIndex)
-            themeIndex = seriesIndex % theme.baseGradients().size();
-        q->setBaseGradient(theme.baseGradients().at(themeIndex));
+    if (theme.seriesGradients().size() && (force || !m_themeTracker.baseGradientOverride)) {
+        if (theme.seriesGradients().size() <= seriesIndex)
+            themeIndex = seriesIndex % theme.seriesGradients().size();
+        q->setBaseGradient(theme.seriesGradients().at(themeIndex));
         m_themeTracker.baseGradientOverride = false;
     }
     if (force || !m_themeTracker.singleHighlightColorOverride) {
@@ -911,7 +1127,7 @@ void QAbstract3DSeriesPrivate::setItemLabelVisible(bool visible)
 
 bool QAbstract3DSeriesPrivate::isUsingGradient()
 {
-    return m_colorStyle != Q3DTheme::ColorStyle::Uniform ? true : false;
+    return m_colorStyle != QGraphsTheme::ColorStyle::Uniform ? true : false;
 }
 
 QT_END_NAMESPACE
