@@ -25,6 +25,7 @@ GraphModifier::GraphModifier(Q3DSurfaceWidgetItem *graph)
       m_series2(new QSurface3DSeries),
       m_series3(new QSurface3DSeries),
       m_series4(new QSurface3DSeries),
+      m_lineSeries(new QSurface3DSeries),
       m_gridSliderX(0),
       m_gridSliderZ(0),
       m_axisRangeSliderX(0),
@@ -36,7 +37,7 @@ GraphModifier::GraphModifier(Q3DSurfaceWidgetItem *graph)
       m_activeSample(0),
       m_fontSize(40),
       m_rangeX(34.0),
-      m_rangeY(16.0),
+      m_rangeY(20.0),
       m_rangeZ(34.0),
       m_minX(-17.0),
       m_minY(-8.0),
@@ -50,6 +51,7 @@ GraphModifier::GraphModifier(Q3DSurfaceWidgetItem *graph)
       m_drawMode2(QSurface3DSeries::DrawSurfaceAndWireframe),
       m_drawMode3(QSurface3DSeries::DrawSurfaceAndWireframe),
       m_drawMode4(QSurface3DSeries::DrawSurfaceAndWireframe),
+      m_drawMode5(QSurface3DSeries::DrawSurface),
       m_offset(4.0f),
       m_ascendingX(true),
       m_ascendingZ(true)
@@ -97,6 +99,8 @@ GraphModifier::GraphModifier(Q3DSurfaceWidgetItem *graph)
     fillSeries();
     changeStyle();
 
+    m_lineSeries->setDrawMode(m_drawMode5);
+
     m_theSeries->setItemLabelFormat(QStringLiteral("@seriesName: (@xLabel, @zLabel): @yLabel"));
 
     connect(&m_timer, &QTimer::timeout, this, &GraphModifier::timeout);
@@ -163,6 +167,27 @@ void GraphModifier::fillSeries()
     m_multiseries[1]->dataProxy()->resetArray(dataArray2);
     m_multiseries[2]->dataProxy()->resetArray(dataArray3);
     m_multiseries[3]->dataProxy()->resetArray(dataArray4);
+
+    QSurfaceDataArray lineDataArray;
+    lineDataArray.reserve(m_zCount);
+    for (int i = 0; i < m_zCount; i++) {
+        QSurfaceDataRow row;
+        float zAdjust = 0.0f;
+        float z = float(i) - m_limitZ + 0.5f + m_multiSampleOffsetZ[3] + zAdjust;
+
+        int j = m_xCount / 2;
+        float xAdjust = 0.0f;
+        if (j == 4)
+            xAdjust = 0.7f;
+        float x = float(j) - m_limitX + 0.5f + m_multiSampleOffsetX[3] + xAdjust;
+        float angle = (z * x) / full * 1.57f;
+        float y = (qSin(angle * float(qPow(1.3f, 4))) + 1.1f * 4) * 3.0f - 5.0f + xAdjust + zAdjust;
+        row.append(QSurfaceDataItem(x,y,z));
+
+        lineDataArray << row;
+    }
+
+    m_lineSeries->dataProxy()->resetArray(lineDataArray);
 }
 
 void GraphModifier::toggleSeries1(int enabled)
@@ -209,6 +234,15 @@ void GraphModifier::toggleSeries4(int enabled)
     }
 }
 
+void GraphModifier::toggleLineSeries(int enabled)
+{
+    qDebug() << __FUNCTION__ << " enabled = " << enabled;
+    if (enabled)
+        m_graph->addSeries(m_lineSeries);
+    else
+        m_graph->removeSeries(m_lineSeries);
+}
+
 void GraphModifier::toggleSmooth(int visible)
 {
     qDebug() << "GraphModifier::toggleSmooth " << visible;
@@ -259,6 +293,20 @@ void GraphModifier::toggleSeriesVisible(int enable)
 #endif
 }
 
+void GraphModifier::toggleFill(int enable)
+{
+    qDebug() << "GraphModifier::toggleFill" << enable;
+    if (enable)
+        m_drawMode |= QSurface3DSeries::DrawFilledSurface;
+    else
+        m_drawMode &= ~QSurface3DSeries::DrawFilledSurface;
+
+    m_theSeries->setDrawMode(m_drawMode);
+#ifdef MULTI_SERIES
+    m_multiseries[0]->setDrawMode(m_drawMode);
+#endif
+}
+
 void GraphModifier::toggleSmoothS2(int visible)
 {
     qDebug() << __FUNCTION__ << visible;
@@ -298,6 +346,20 @@ void GraphModifier::toggleSeries2Visible(int enable)
     m_multiseries[1]->setVisible(enable);
 }
 
+void GraphModifier::toggleFillS2(int enable)
+{
+    qDebug() << "GraphModifier::toggleFill" << enable;
+    if (enable)
+        m_drawMode2 |= QSurface3DSeries::DrawFilledSurface;
+    else
+        m_drawMode2 &= ~QSurface3DSeries::DrawFilledSurface;
+
+    m_theSeries->setDrawMode(m_drawMode2);
+#ifdef MULTI_SERIES
+    m_multiseries[1]->setDrawMode(m_drawMode2);
+#endif
+}
+
 void GraphModifier::toggleSmoothS3(int enabled)
 {
     qDebug() << __FUNCTION__ << enabled;
@@ -337,6 +399,20 @@ void GraphModifier::toggleSeries3Visible(int visible)
     m_multiseries[2]->setVisible(visible);
 }
 
+void GraphModifier::toggleFillS3(int enable)
+{
+    qDebug() << "GraphModifier::toggleFill" << enable;
+    if (enable)
+        m_drawMode3 |= QSurface3DSeries::DrawFilledSurface;
+    else
+        m_drawMode3 &= ~QSurface3DSeries::DrawFilledSurface;
+
+    m_theSeries->setDrawMode(m_drawMode3);
+#ifdef MULTI_SERIES
+    m_multiseries[2]->setDrawMode(m_drawMode3);
+#endif
+}
+
 void GraphModifier::toggleSmoothS4(int visible)
 {
     qDebug() << __FUNCTION__ << visible;
@@ -370,12 +446,49 @@ void GraphModifier::toggleSurfaceS4(int enable)
     m_multiseries[3]->setDrawMode(m_drawMode4);
 }
 
+void GraphModifier::toggleLineSurfaceGrid(int enable)
+{
+    qDebug() << __FUNCTION__ << enable;
+    if (enable)
+        m_drawMode5 |= QSurface3DSeries::DrawWireframe;
+    else
+        m_drawMode5 &= ~QSurface3DSeries::DrawWireframe;
+
+    m_lineSeries->setDrawMode(m_drawMode5);
+
+}
+
 void GraphModifier::toggleSeries4Visible(int enable)
 {
     qDebug() << __FUNCTION__ << enable;
     m_multiseries[3]->setVisible(enable);
 }
 
+void GraphModifier::toggleFillS4(int enable)
+{
+    qDebug() << "GraphModifier::toggleFill" << enable;
+    if (enable)
+        m_drawMode4 |= QSurface3DSeries::DrawFilledSurface;
+    else
+        m_drawMode4 &= ~QSurface3DSeries::DrawFilledSurface;
+
+    m_theSeries->setDrawMode(m_drawMode);
+#ifdef MULTI_SERIES
+    m_multiseries[3]->setDrawMode(m_drawMode4);
+#endif
+}
+
+void GraphModifier::toggleFillLine(int enable)
+{
+    qDebug() << "GraphModifier::toggleFill" << enable;
+    if (enable)
+        m_drawMode5 |= QSurface3DSeries::DrawFilledSurface;
+    else
+        m_drawMode5 &= ~QSurface3DSeries::DrawFilledSurface;
+
+    m_lineSeries->setDrawMode(m_drawMode5);
+}
+
 void GraphModifier::toggleSqrtSin(int enable)
 {
     if (enable) {
@@ -633,10 +746,10 @@ void GraphModifier::gradientPressed()
     }
 
     QLinearGradient gradient;
-    gradient.setColorAt(0.0, Qt::black);
-    gradient.setColorAt(0.33, Qt::blue);
-    gradient.setColorAt(0.67, Qt::red);
-    gradient.setColorAt(1.0, Qt::yellow);
+    gradient.setColorAt(0.0, QColor(0, 0, 0, 255));
+    gradient.setColorAt(0.33, QColor(0, 0, 255, 100));
+    gradient.setColorAt(0.67, QColor(255, 0, 0, 150));
+    gradient.setColorAt(1.0, QColor(255, 255, 0, 200));
 
     QList<QLinearGradient> gradients;
     gradients << gradient;
@@ -943,6 +1056,12 @@ void GraphModifier::changeShadowQuality(int quality)
     m_graph->setShadowQuality(sq);
 }
 
+void GraphModifier::changeTransparencyTechnique(int technique)
+{
+    QtGraphs3D::TransparencyTechnique sq = QtGraphs3D::TransparencyTechnique(technique);
+    m_graph->setTransparencyTechnique(sq);
+}
+
 void GraphModifier::changeSelectionMode(int mode)
 {
     QComboBox *comboBox = qobject_cast<QComboBox *>(sender());
@@ -1681,44 +1800,44 @@ void GraphModifier::changeMesh()
     static int model = 0;
     switch (model) {
     case 0:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Cylinder);
-        m_graph->seriesList().at(0)->setMeshSmooth(false);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Cylinder);
+        m_graph->selectedSeries()->setMeshSmooth(false);
         break;
     case 1:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Cylinder);
-        m_graph->seriesList().at(0)->setMeshSmooth(true);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Cylinder);
+        m_graph->selectedSeries()->setMeshSmooth(true);
         break;
     case 2:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Cone);
-        m_graph->seriesList().at(0)->setMeshSmooth(false);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Cone);
+        m_graph->selectedSeries()->setMeshSmooth(false);
         break;
     case 3:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Cone);
-        m_graph->seriesList().at(0)->setMeshSmooth(true);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Cone);
+        m_graph->selectedSeries()->setMeshSmooth(true);
         break;
     case 4:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Bar);
-        m_graph->seriesList().at(0)->setMeshSmooth(false);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Bar);
+        m_graph->selectedSeries()->setMeshSmooth(false);
         break;
     case 5:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Bar);
-        m_graph->seriesList().at(0)->setMeshSmooth(true);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Bar);
+        m_graph->selectedSeries()->setMeshSmooth(true);
         break;
     case 6:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Pyramid);
-        m_graph->seriesList().at(0)->setMeshSmooth(false);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Pyramid);
+        m_graph->selectedSeries()->setMeshSmooth(false);
         break;
     case 7:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::Pyramid);
-        m_graph->seriesList().at(0)->setMeshSmooth(true);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::Pyramid);
+        m_graph->selectedSeries()->setMeshSmooth(true);
         break;
     case 8:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::BevelBar);
-        m_graph->seriesList().at(0)->setMeshSmooth(false);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::BevelBar);
+        m_graph->selectedSeries()->setMeshSmooth(false);
         break;
     case 9:
-        m_graph->seriesList().at(0)->setMesh(QAbstract3DSeries::Mesh::BevelBar);
-        m_graph->seriesList().at(0)->setMeshSmooth(true);
+        m_graph->selectedSeries()->setMesh(QAbstract3DSeries::Mesh::BevelBar);
+        m_graph->selectedSeries()->setMeshSmooth(true);
         break;
     }
     model++;
